;*****************************************************************************************
;  RGB Background Color Fader
;  This little jammy was made for fun to see how hard it was to do (it wasn't hard..)
;  It does 2 things.
; First, It can fade the current background colors to 1 color that you define. You can
; fade to blue, gray, pink, black, etc.
;
; Second you can fade from 1 color that you define to a palette that is stored in memory.
; You can fade from white to a nice picture, or go from all blue to a black and white picture.
; What ever you want, just set up the Red, Green, Blue colors and have the palette somewhere
; in memory.
;
; You can write extra code to handle the sprites (it's the same way, just read/write the
; sprite colors instead of the background colors.)
;
;  - Fades Background Colors (not sprite colors) to 1 color (you can make it any color!)
;
;  - Fades from 1 color to a pre-defined palette in memory
;
;  In this demo:
;  Press A to fade from palette to black
;  Press B to fade from white to palette
;
; How you can use it:
;  enter fade type/ counter/ and color in Red, Green, and Blue values then call
; Fade routine.  It runs in a Vertical Blank so you can still have music, effects, etc
; while it fades.
;
; fadecolors is a 64 byte table in ram, when fading this table contains the current
; colors that are on the screen.
;
; colors is a table in ROM that contains the colors which are on the screen
;*****************************************************************************************

SECTION "Main Mem", BSS[$C000]

fadetype: ds 1; 0 = fadeout 1 = fadein
fadeon: ds 1; 0 = no fade, 1 = fade
fadecount: ds 1; 0 = no more colors to fade
fadecolors: ds 64; Background color table   32 colors, 2 bytes each
redstore: ds 1
greenstore: ds 1
bluestore: ds 1
rowcounter: ds 1
joyold: ds 1
joynew: ds 1
fadered: ds 1       ; fade to this red level
fadegreen: ds 1     ; fade to this green level
fadeblue: ds 1      ; fade to this blue level




;*** Vertical blanking IRQ ***
SECTION "Vertical blank IRQ",home[$0040]
 jp vbint





    ;******************************************************************
    ;                            Header Equates
    ;******************************************************************
;Cartridge type:
; 0 - Rom only      3 - ROM+MBC1+RAM+Battery  1 - ROM+MBC1      5 - ROM+MBC2
; 2 - ROM+MBC1+RAM  6 - ROM+MBC2+Battery
CARTTYPE  EQU 2
;Rom Size:
; 0 - 256kBit =  32kB =  2 banks ; 1 - 512kBit =  64kB =  4 banks
; 2 -   1MBit = 128kB =  8 banks ; 3 -   2MBit = 256kB = 16 banks
; 4 -   4MBit = 512kB = 32 banks
ROMSIZE   EQU 0
;Ram Size:
; 0 - None ; 1 -  16kBit =  2kB = 1 bank
; 3 - 256kBit = 32kB = 4 banks
RAMSIZE   EQU 3
;Manufacturer code (word):
MANUCODE  EQU 1
;Version Number (byte):
VERSION   EQU 1

;************************************************************************************
;     Do NOT modify unless you are familiar with header info *Modified for Game Boy Color*
;************************************************************************************

SECTION "Header",home[$0100]
  ;Jumper
  nop
  jp Start ; go to Start of program:
  ;Nintendo Scrolling Title Graphic
  DB $CE,$ED,$66,$66,$CC,$0D,$00,$0B,$03,$73,$00,$83,$00,$0C,$00,$0D
  DB $00,$08,$11,$1F,$88,$89,$00,$0E,$DC,$CC,$6E,$E6,$DD,$DD,$D9,$99
  DB $BB,$BB,$67,$63,$6E,$0E,$EC,$CC,$DD,$DC,$99,$9F,$BB,$B9,$33,$3E
  ;Title of the game
  db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128 ; Filled in by RGBFIX
  ;Not used
  db 0,0,0; last should be 3, won't work in emu unles it's 0
  ;Cartridge type:
  db CARTTYPE
  ;Rom Size:
  db ROMSIZE
  ;Ram Size:
  db RAMSIZE
  ;Manufacturer code:
  db 00, $33
  ;Version Number
  db VERSION
  ;Complement check
  db $0a
  ;Checksum
  dw 0





  ;*********************************************************************
  ;          This is the start of the program
  ;*********************************************************************

 ;SECTION "Main", home

Start:


  ld  SP,$FFFD          ; Init stack ptr

  di                    ; disable interrupts

  ld a, 0
  ldio  [$42],a         ; Y Scroll=0
  ldio  [$43],a         ; X Scroll = 0

  ld  a,%00000000       ; Turn off LCD Controller
  ldio [$40],a

  call copygfx          ; copy graphics to vram
  call copytiles        ; copy tiles to vram


  ld a,$80              ; set the palette at 0 and allow it to auto-increase
  ldio [$68], a

  ld hl, colors         ; get address of color table (reads 32 colors)

  ld d, 64              ; # number of colors to copy * 2
copycolors:
  ld  a, [hl+]          ; get data, increase address
  ldio [$69], a         ; put the data into the palette
  dec d
  jr nz, copycolors



  ld     a,%10010001    ; LCD Controller = On
                        ; BG Gfx = $8000
                        ; BG Tiles   = $9800
                        ; BG     = On
  ldio    [$40],a





  ld a,%00000001        ; Allow LCDC Status Interrupt & Vertical Blank
  ldio  [$FF],a


  xor a
  ld [fadetype], a   ; 0 = fadeout 1 = fadein
  ld [fadeon], a    ; 0 = no fade, 1 = fade
  ld [fadecount], a ; 0 = no more colors to fade

  ei                     ; enable interrupts



;========================================================================
;                       Don't do anything important
;========================================================================

megaloop:
  halt
  nop
                     ; Do Nothing
  jr megaloop        ; Do it all over again!




;===========================================================================================
;                              Vertical Blank Interrupt
;===========================================================================================
vbint:
   push AF
   push BC
   push DE             ; typical register saver stuff for interrupts
   push HL

   ld a, [fadeon]      ; if fade is turned off then don't fade colors
   or a
   jp z, fadetest      ; if it's off then check the Joypad for the next command

   call faderoutine    ; if fade is on then do fade
  ; if the screen is fading then don't read joypad

   jp skipfade

 ;====================
fadetest:                ; test joypad for buttons and do their bidding
   call joypad          ; read joypad

   ld a, [joyold]
   cp 1                 ; was A button pressed?
   jp z, initfadetocolor; if A pressed then start Fade To Color
.l1:
   cp 2                  ; was B button press?
   jp z, initfadetopalette  ; If B pressed then start Fade To Palette

   jp skipfade


;=====================================
; A button pressed, set up fade to color
; This routine copies the current palette from the GB and stores it in ram.
; Then it sets up counter values, turns the fader on, and enters the color to fade to
initfadetocolor:

   ld b, 0

  ld hl, fadecolors         ; get address of fade color table
  ld d, 64              ; # number of colors to copy * 2
.l2:

 ld a, b
 ldio [$68], a          ; backup BG Colors
 ldio a, [$69]
 ld [hli], a

  inc b
  dec d
  jr nz, .l2

  ld a, 1
  ld [fadeon], a        ; enable fading

  xor a                 ; a = 0
  ld [fadetype], a      ; fade from palette to 1 color

  ld a, $0
  ld [fadered], a       ; Red = 0; green = 0; blue = 0
  ld [fadegreen], a     ; that equals Black!
  ld [fadeblue], a

  ld a, 33
  ld [fadecount], a     ; 32 levels of RGB, 32 maximum decreases, add 1 so the last
                        ; fade palette will be entered into the GB
                        ;
  jp skipfade           ; quit to end of Vertical Blank interrupt


;=============================================================================
; Set up a fade from 1 color to a palette

initfadetopalette:
  ld a, $1f
  ld [fadered], a        ; Create a White color
  ld [fadegreen], a      ; you can change these values so you may fade from a different
  ld [fadeblue], a       ; color


        ld hl, fadecolors         ; get address of fade color table
        ld d, 32              ; # number of colors to copy * 2
.l2:
        push de
        ld a, [fadered]
        ld b, a
        ld a, [fadegreen]      ; set this color up as the palette
        ld c, a
        ld a, [fadeblue]
        ld d, a
        sla c                   ; convert the R,G,B, to the 15 bit value GB understands
        sla c
        sla c

        sla c
        rl  d
        sla c
        rl d

        ld a, b
        or a, c
        ld [hl+], a

        ld a, d
        ld [hl+], a
        pop de
        dec d
        jr nz, .l2

  ld a, 1
  ld [fadeon], a         ; enable fading
  ld a, 1
  ld [fadetype], a      ; fade from 1 color to palette

  ld a, 33
  ld [fadecount], a     ; 32 levels of RGB, 32 maximum decreases  add 1 so the last
                        ; fade will be put into the GB
  ;jr skipfade           ; exit VB interrupt

skipfade:

   pop  HL
   pop  DE
   pop  BC    ; restore register values
   pop  AF
   reti


;====================================================================================
;   Fade Routine - Fade out, Fade in
;=====================================================================================

faderoutine:
        ld a, [fadetype]             ; do we fade to a single color?
        or a
        jp z, fadetocolor            ; Yup! Go to fade to c olor routine

fadetopalette:                       ; or do we fade from one color to a full palette?

        ; update GB palette from temporary palette in RAM

        ld a,$80              ; set the palette at 0 and allow it to auto-increase
        ldio [$68], a

        ld hl, fadecolors         ; get address of color table (reads 32 colors)

        ld d, 64              ; # number of colors to copy * 2
.l1:

        ld  a, [hl+]          ; get data, increase address
        ldio [$69], a         ; put the data into the palette
        dec d
        jr nz, .l1

        ld hl, fadecolors
        ld c, 32

fade_to_palette_loop:
        ld a, [hl+]    ; get one color from table and store it in de
        ld e, a
        ld a, [hl+]
        ld d, a
        dec hl
        dec hl

; fade to red level

        push bc
        push hl
        ld hl, colors     ; get a color from a palette stored in memory
        ld a, c
        dec a
        xor $ff
        and $1f
        sla a
        ld c, a
        ld b, 0
        add hl, bc        ; fetch red value
        ld a, [hl]
        and $1f
        ld [fadered], a
        pop hl
        pop bc



        ld a, [fadered]
        ld b, a

        ld a, e           ; get red value from current GB palette
        and $1f
        cp b              ; compare with one in memory
        jr z, .l1         ; if the colors are equal then save it
        jr c, .l11
        dec a              ; if it's greater than then decrease the number
        jr .l1
.l11:
        inc a              ; if it's less then then increase the number

.l1:
        ld [redstore], a   ; temporarily save it


; fade to green level

        ld a, d
        sra a
        rr e
        sra a
        rr e                ; get green value from GB palette
        sra a
        rr e
        sra a
        rr e
        sra a
        rr e

        push bc
        push de
        push hl
        ld hl, colors
        ld a, c
        dec a
        xor $ff
        and $1f
        sla a
        ld c, a
        ld b, 0           ; get green value from palette in memory
        add hl, bc
        ld a, [hl+]
        ld e, a
        ld a, [hl]
        ld d, a

        ld a, d
        sra a
        rr e
        sra a
        rr e
        sra a
        rr e
        sra a
        rr e
        sra a
        rr e
        ld a, e
        and $1f
        ld [fadegreen], a
        pop hl
        pop de
        pop bc



        ld a, [fadegreen]
        ld b, a

        ld a, e
        and $1f
        cp b
        jr z, .l2  ; if the colors are equal then save it
        jr c, .l22
        dec a
        jr .l2
.l22:
        inc a

.l2:
        ld [greenstore], a

; fade to blue level


        push bc
        push hl
        ld hl, colors
        ld a, c
        dec a
        xor $ff
        and $1f               ; get blue value from palette in memory
        sla a
        ld c, a
        ld b, 0
        add hl, bc
        inc hl
        ld a, [hl]
        sra a
        sra a
        and $1f

        ld [fadeblue], a
        pop hl
        pop bc


        ld a, [fadeblue]
        ld b, a

        ld a, d            ; get blue value from GB palette
        sra a
        sra a
        and $1f
        cp b
        jr z, .l3  ; if the colors are equal then save it
        jr c, .l33
        dec a
        jr .l3
.l33:
        inc a

.l3:
        ld [bluestore], a


        push bc

        ld a, [redstore]
        ld b, a
        ld a, [greenstore]
        ld c, a
        ld a, [bluestore]
        ld d, a
        sla c
        sla c                  ; combine new Red, Green, Blue values and store them
        sla c                  ; into ram again

         sla c
        rl  d
        sla c
         rl d

         ld a, b
         or a, c
        ld [hl+], a

         ld a, d
         ld [hl+], a
        pop bc
        dec c
        ld a, c
        or a
        jp nz, fade_to_palette_loop

        ld a, [fadecount]
        dec a
        ld [fadecount], a       ; if 32 levels of fade has been done then
        or a
        ret nz

        xor a
        ld [fadeon], a          ; turn off fade



        ret




;================================================================
; Fade from a palette to a single color

fadetocolor:
        ; update GB palette from temporary palette in RAM

  ld a,$80              ; set the palette at 0 and allow it to auto-increase
  ldio [$68], a

  ld hl, fadecolors         ; get address of color table (reads 32 colors)

  ld d, 64
.l1:


  ld  a, [hl+]          ; get data, increase address
  ldio [$69], a         ; put the data into the palette
  dec d
  jr nz, .l1




        ld hl, fadecolors
        ld c, 32

fadeoutloop:
        ld a, [hl+]    ; get one color from table and store it in de
        ld e, a
        ld a, [hl+]
        ld d, a
        dec hl
        dec hl

; fade to red level
        ld a, [fadered]
        ld b, a

        ld a, e       ; match red from palette to the one that was specified
        and $1f
        cp b
        jr z, .l1  ; if the colors are equal then save it
        jr c, .l11
        dec a
        jr .l1
.l11:
        inc a

.l1:
        ld [redstore], a



        ld a, d
        sra a
        rr e
        sra a
        rr e
        sra a
        rr e
        sra a
        rr e
        sra a
        rr e

; fade to green level
        ld a, [fadegreen]
        ld b, a

        ld a, e      ; match green from palette to the one that was specified

        and $1f
        cp b
        jr z, .l2  ; if the colors are equal then save it
        jr c, .l22
        dec a
        jr .l2
.l22:
        inc a

.l2:
        ld [greenstore], a

; fade to blue level

        ld a, [fadeblue]
        ld b, a

        ld a, d
        sra a
        sra a
        and $1f     ; match blue from palette to the one that was specified

        cp b
        jr z, .l3  ; if the colors are equal then save it
        jr c, .l33
        dec a
        jr .l3
.l33:
        inc a

.l3:
        ld [bluestore], a


        push bc

        ld a, [redstore]
        ld b, a
        ld a, [greenstore]
        ld c, a
        ld a, [bluestore]
        ld d, a
        sla c           ; combine Red, Green, Blue values to a 15 bit number GB understands
        sla c
        sla c

         sla c
        rl  d
        sla c
         rl d

         ld a, b
         or a, c
        ld [hl+], a         ; save the low byte of the number into RAM

         ld a, d
         ld [hl+], a        ; save the high byte of the number into RAM
        pop bc
        dec c
        ld a, c
        or a
        jp nz, fadeoutloop

        ld a, [fadecount]
        dec a
        ld [fadecount], a       ; if 32 levels of fade has been done then
        or a
        ret nz

        xor a
        ld [fadeon], a          ; turn off fade



        ret

;***********************************************************************************
;                        Read Joypad and return values in B
;***********************************************************************************
joypad:
        LD A,$20      ; <- bit 5 = $20
        LDIO [$00],A  ; <- select P15 by setting it low
        LDIO A,[$00]
        LDIO A,[$00]  ; <- wait a few cycles
        CPL           ; <- complement A
        AND $0F       ; <- get only first 4 bits
        SWAP A        ; <- swap it
        LD B,A        ; <- store A in B
        LD A,$10
        LDIO [$00],A  ; <- select P14 by setting it low
        LDIO A,[$00]
        LDIO A,[$00]
        LDIO A,[$00]
        LDIO A,[$00]
        LDIO A,[$00]
        LDIO A,[$00]  ; <- Wait a few MORE cycles
        CPL           ; <- complement (invert)
        AND $0F       ; <- get first 4 bits
        OR B          ; <- put A and B together

        LD B,A        ; <- store A in B
        LD   A,[joynew]  ; <- read old joy data from ram
        XOR B         ; <- toggle w/current button bit
        AND B         ; <- get current button bit back
        LD   [joyold],A  ; <- save in new Joydata storage
        LD A,B        ; <- put original value in A
        LD   [joynew],A  ; <- store it as old joy data


        LD A,$30      ; <- deselect P14 and P15
        LDIO [$00],A  ; <- RESET Joypad
        RET           ; <- Return from Subroutine

        ; The button values using the above method are such:
        ; read [joyold] to get the joy values:
        ;
        ;  $8 - Start             $80 - Down
        ;  $4 - Select            $40 - Up
        ;  $2 - B                 $20 - Left
        ;  $1 - A                 $10 - Right



;***************************************************************
;*                          Copy Tile Gfx
;***************************************************************

copygfx:
 ld  bc,colgfx
 ld  hl,$8000
 ld  de, 16          ;# of bytes to copy

copyloop:
 ldio  a,[$41]       ; Wait until you can write to vram
 and 2
 jr  nz, copyloop

 ld  a, [bc]
 ld  [hli],a
 inc  bc
 dec  de
 ld a,d
 cp $ff
 jp  nz,copyloop
 ret



;***************************************************************
;*                          Copy Tile Map
;***************************************************************

copytiles:

  ld  hl,$9800
  ld  de, 1024          ;# of bytes to erase
eraseloop:

 ldio  a,[$41]          ; Wait until you can write to vram
 and  2
 jr nz,eraseloop
 ld  a,$00

 ld  [hli],a
 dec  de
 ld  a,d
 cp  $ff
 jr  nz,eraseloop


  ld a, 1
  ldio [$4f], a          ; switch to VRAM bank 1 so we can add palette and x flip data!

  ld  bc, paletternd     ; get address of palette data
  ld  a, 0
  ld  [rowcounter], a    ; # of rows that were made
  ld  hl,$9800           ; vram address
  ld  d, $00             ; # of columns that were made
createloop:

  ldio  a,[$41]          ; Wait until you can write to vram
  and  2
  jr nz,createloop

  ld a, d                ; get column
  and 1                  ; if it's the right side of the column the we need to x-flip the tile
  scf
  ccf
  rl a                   ; take that bit and shift it 5 times to make it the X-flip bit
  rl a
  rl a
  rl a
  rl a

  ld e, a                ; store it in E
  ld  a, [bc]            ; get palette #
  or e                   ; add the x-flip bit
  ld  [hli],a            ; put it in Vram

  ld a, d                ; get column
  and 1
  jr z, colornevermind   ; did we do the right side of the column? (each column is 2 tiles
                         ; made up of 1 tile, the right one is flipped horizontally)
                         ; if we didn't make the right side yet then don't change the palette
                         ; address!
  inc bc                 ; increase palette address
colornevermind:
  inc d                  ; increase the column
  ld a, d
  cp 32                  ; compare it with 32 (number of tiles to be made)
  jp nz, createloop      ; if not then go back
  ld d, 0                ; if so then reset column to 0
  ld bc, paletternd      ; reset palette address to beginning
  ld a, [rowcounter]
  inc a                  ; increase the row we're on
  ld [rowcounter], a
  cp 32                  ; see if it's 32 rows
  jr nz, createloop      ; if not the go back again


  ld a, 0
  ldio [$4f], a          ; switch to VRAM bank 1

 ret




;=============================================================================
;                          The rest is all data
;=============================================================================


paletternd:
db  7,  1,  3,  2,  0,  4,  5,  6
db  5,  7,  1,  2,  6,  4,  3,  0





colgfx:                                    ; just one character (1 tile 8*8)
DB $EE,$F0,$C4,$F8,$EE,$F0,$C4,$F8
DB $EE,$F0,$C4,$F8,$EE,$F0,$C4,$F8

;================================
; This is the palette we fade to when you press B
colors:
DB $FF,$7F, $9F,$52, $5F,$29, $1F,$00, $FF,$7F, $94,$7E, $4A,$7D, $00,$7C
DB $FF,$7F, $94,$53, $20,$2B, $60,$02, $FF,$7F, $7D,$53, $1B,$2B, $9A,$02
DB $FF,$7F, $9F,$7E, $5F,$7D, $1F,$7C, $FF,$7F, $74,$6F, $0A,$5F, $80,$52
DB $FF,$7F, $7B,$6F, $F7,$5E, $94,$52, $FF,$7F, $9F,$6A, $5F,$55, $1F,$44

